#TITLE   Joint Maximization
option presolve 0;
###################################################################
###################################################################
#                    MODEL DEFINITION                             #
###################################################################
###################################################################

#Declaration/Initialisation of User Definad Sets
###################################################################

set RG ordered			;		#REGIONS

set YR := 1990..2200 by 10;			#ALL YEARS
set NOYR := 2130..2200 by 10;			#LAST YEARS
set TP:= YR diff NOYR ordered;			#TIME PERIODS (STARTING YEAR)
set PP:={t in TP: ord(t)>1} ordered by TP;	#PROJECT PER. (ALL BUT BASE PER)

set FYset := {"FY"}	;
set TRD			;			#TRADEABLE GOODS


#Declaration and definition of parameters
###################################################################
#NEGISHI ITERATIONS
param nITER integer :=4; 			# number of Negishi iterations
param ITER integer<=nITER;
param NWTITR{1..nITER,RG};
param time_begin integer;
param time_end integer;
param sumNW;
param INF := 1e5; 				#infinity
param INIT_ZC{t in TP}:=  			#LOWER BOUND and IN VALUE OF ZC 
	if(ord(t)==1) then 0 else 1;
param FYR{t in TP, fyr in FYset} ; 		#FIRST YEAR OF TIME PERIOD
param TE{t in TP} := 				#YEARS SINCE BEG OF 1st PER
	FYR[t,"FY"] - FYR[first(TP),"FY"]; 
param NYPER{t in TP} := 			#NB YEARS IN A PER-FORWARD DIF
      if(ord(t)<card(TP)) then TE[next(t)]-TE[t]
	          	  else NYPER[prev(t)];
param GDP{r in RG} ;				#INITIAL GDP ($ TRILLIONS)
param KGDP{r in RG} ; 				#INITIAL CAPITAL-GDP RATIO
param DEPR{r in RG} ; 				#ANNUAL DEPREC RATE OF CAPITAL
param KPVS{r in RG} ; 				#CAPITAL VALUE SHARE OF GDP
param PGROW{t in YR, r in RG} ;	 		#POTENTIAL GDPGROWTHRATES-ANN%
param GROW{t in TP,r in RG} := PGROW[t,r]/100 ;	#ANNUAL LABOR FORCE GROWTH RATE
param SCC := 500; 				#STABILIZE CUMULCO2 EMISSIONS
						# BILLION TONS
param ZBASE{t in YR, r in RG} ;	 		#BAU CARBON EMISSIONS LEVELS
param INIT_Z{t in TP, r in RG}:=  		#Fixed and initial values of Z 
   if(ord(t)==card(TP)) 
      then 0 else ZBASE[first(TP),r];
param AQPER{t in YR, r in RG} ;	 		#MAXIMUM LEVEL OF QUADRATIC 
						#ABATEMENT - PERCENTAGE OF BAU
param ACB{r in RG,p in PP} := 0.2;		#ABATEMENT COST PARAM-BACKSTOP
param ACQ{r in RG,p in PP} := 			#ABATEMENT COST PARAMETER-QUAD
    (0.5*ACB[r,p])/(0.01*AQPER[p,r]*ZBASE[p,r]);
param KSUR{t in TP,r in RG}:= 			#CAPITAL SURVIVAL FACTOR
	(1-DEPR[r])**NYPER[t];
param LBVS{r in RG}:= 1 - KPVS[r];		#LABOR VALUE SHARE OF GDP
param NW{r in RG} >= 0			;	#NEGISHI WEIGHTS
param UDR{r in RG,t in TP} :=  			#UTILITY DISCOUNTRATE FOR PER TP
	KPVS[r]/KGDP[r] - DEPR[r] - GROW[t,r];	
param L {r in RG,t in TP} :=			#CURRENT LABOR FORCE(EFF. UNITS)
	if (ord(t)>1) 
		then L[r,prev(t)]*((1+GROW
                    [prev(t),r])**NYPER[prev(t)])
		else 1;
param UDFT{r in RG,t in TP} :=			#UTILITY DISCOUNTFACTOR FORPERTP 
  if (ord(t)<card(TP))
    then
	if (ord(t)==1)
	then 1
	else UDFT[r,prev(t)]*
            ((1-UDR[r,prev(t)])**NYPER[prev(t)])
    else
        UDFT[r,prev(t)]*((1-UDR[r,prev(t)])**NYPER[prev(t)]) / (1-(1-UDR[r,t])**NYPER[t]);

param UDFSUM{r in RG}:=  			#UDF NORMALIZING CONSTANT
	sum{p in PP} UDFT[r,p];
param UDF{r in RG,p in PP}:= UDFT[r,p]/UDFSUM[r];

param K0{r in RG}:=KGDP[r]*GDP[r];		#INITIAL CAPITAL
param I0{r in RG}:=				#INITIAL INVESTMENT
	K0[r]*(GROW[first(TP),r]+DEPR[r]);
param C0{r in RG} 	:=  GDP[r] - I0[r];	#INITIAL CONSUMPTION

#INITIAL GDP
param Y0{r in RG} 	:=  GDP[r];

#PRODUCTION FUNCTION SCALE FACTOR
param A{r in RG} 	:= Y0[r]/(K0[r]**KPVS[r]);

param SH{p in YR, r in RG} ; 			#SHARES IN GLOBAL CO2 EMISSIONS

#DEBUGGING: new parameters allowing the run of first Negishi iteration

param PVPI{a in TRD,p in PP} ;			#PRESENT VALUE EXPECTED PRICE OF TRADEABLE GOODS

#The 2000 shares are proportional to 1990 regional emissions.
#The 2030 shares (and thereafter) are proportional to 1990 populations.
#The shares between 2000 and 2030 are determined by linear interpolation.
#Shares must sum to unity in each time period.
#The population figures used are shown below (in billions).
#           USA     OECD   USSR   CHINA    ROW
#1990      0.250   0.582   0.289   1.116   3.024


param XLOBND{r in RG,a in TRD,t in TP}:=
if (ord(t)==1) then 0 else -INF ;	#lobnd of X
param XUPBND{r in RG,a in TRD,t in TP}:=
if (ord(t)==1) then 0 else INF ;	#upbnd of X
param ZUPBND{r in RG,t in TP}:= if (ord(t)==card(TP)) then 0 else INF ;#upbnd of Z
#Declaration and definition of variables
###################################################################

var X{r in RG,a in TRD,t in TP} := 0;           #NET EXPORTS OF TRADEABLE GOODS;

var K{r in RG,t in TP} >= K0[r], :=  K0[r];     #CAPITAL STOCK - TRILLION $

var Y{r in RG,t in TP} >= Y0[r], :=  Y0[r];     #CONVENTIONAL GDP - TRILLION $

var C{r in RG,t in TP} >= C0[r], :=  C0[r];     #CONSUMPTION - TRILLION $

var I{r in RG,t in TP} >= I0[r], :=  I0[r];     #INVESTMENT - TRILLION $

var AQ{r in RG,t in TP} >= 0, :=  0;            #QUADR ABAT LEVEL - BILLION TONS

var AB{r in RG,t in TP} >= 0, :=  0;            #BACKSTOP ABAT LEVEL-BILL.TONS

var Z{r in RG,t in TP} >= if(ord(t)==1) then INIT_Z[t,r] else
if(ord(t)==last(TP)) then INIT_Z[t,r] else 0, :=  0;     #REGIONAL CO2 EMISSIONS-BILLTONS

var ZG{t in TP} >= if(ord(t)==1) then sum{r in RG} ZBASE[first(TP),r] else 0, :=0;                         #GLOBAL CO2 EMISSIONS -BILL.TONS

var ZC{t in TP}>=if(ord(t)==1) then  0 else 1, :=if(ord(t)==1) then  0 else 1, <=  SCC; #GLOBAL CUMULATIVE CO2 EMISSIONS BILLION TONS


#Declaration of constraints
#####################################################################

###Central constraints

    #NWELDF           NEGISHI WELFARE OBJECTIVE FUNCTION
    #TRDBAL(TRD,TP)   GLOBAL TRADE BALANCE
    #RGEM(RG,TP)      REGIONAL SHARES IN GLOBAL EMISSIONS
    #CUME(TP)         CUMULATIVE GLOBAL CARBON EMISSIONS

subject to TRDBAL{a in TRD,p in PP}:
     sum{r in RG} X[r,a,p]  =  0;

subject to RGEM{r in RG,p in PP}:
     Z[r,p] + X[r,"CRT",p]  =  SH[p,r] * ZG[p];

subject to CUME{t in TP: ord(t)>1}:
     ZC[t]  =  ZC[prev(t)]  + 0.5 * NYPER[prev(t)] * (ZG[prev(t)] + ZG[t]);


###Regional blocks of constraints

    #CAPACCUM(RG,TP)  CAPITAL ACCUMULATION
    #TC(RG,TP)        TERMINAL CONDITION ON INVESTMENT
    #PRODT(RG,TP)     COBB-DOUGLAS PRODUCTION FUNCTION
    #GG(RG,TP)        GREEN GDP ALLOCATION
    #BEXP(RG,TP)      LIMIT ON BACKSTOP EXPANSION
    #EMIS(RG,TP)      REGIONAL EMISSIONS DEFINITION;

subject to CAPACCUM{r in RG,t in TP: ord(t)>1}:
     K[r,t]  =  KSUR[t,r] * K[r,prev(t)]
             + 0.5 * NYPER[prev(t)] * (KSUR[prev(t),r] * I[r,prev(t)] + I[r,t]);

#subject to TC{r in RG,t in TP}:
subject to TC{r in RG}:
     I[r,last(TP)]  >=  (GROW[last(TP),r] + DEPR[r]) * K[r,last(TP)];

subject to PRODT{r in RG,p in PP}:
     Y[r,p]  <=  A[r] * (K[r,p]**KPVS[r]) * (L[r,p]**LBVS[r]);

subject to GG{r in RG,p in PP}:
     Y[r,p]  >=  C[r,p] + I[r,p] + X[r,"NUM",p]
               + ACQ[r,p]*(AQ[r,p]**2) + ACB[r,p]*AB[r,p];

#Backstop is limited in its initial market share and expansion rate

subject to BEXP{r in RG,t in TP: ord(t)>1}:
     AB[r,t]  <=  0.01 * ZBASE[t,r]  +  2 * AB[r,prev(t)];

subject to EMIS{r in RG,p in PP}:
     Z[r,p]  =  ZBASE[p,r] - AQ[r,p] - AB[r,p];



#OBJECTIVE
###################################################################

minimize NWEL:
       -1000 * 1000 * sum{r in RG} NW[r] * sum{p in PP} (UDF[r,p] * log(C[r,p])) ;

